!---------------------------------------------------------------------------------------
! GRID_TO_NCDF: grid_convert
!---------------------------------------------------------------------------------------
!  FLEXINVERT is free software: you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation, either version 3 of the License, or
!  (at your option) any later version.
!
!  FLEXINVERT is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with FLEXINVERT.  If not, see <http://www.gnu.org/licenses/>.
!
!  Copyright 2017, Rona Thompson
!---------------------------------------------------------------------------------------
!
!> grid_convert
!! 
!! Purpose:    
!!
!---------------------------------------------------------------------------------------

subroutine grid_convert(files, config, nr, timestamp)

  use mod_var
  use mod_settings
  use mod_flexpart
  use mod_dates

  implicit none

  type (files_t),  intent(in out)        :: files
  type (config_t), intent(in out)        :: config
  integer,         intent(in)            :: nr
  integer,         intent(in)            :: timestamp

  character(max_path_len)                                :: filename, filedates, path_flexpart, path_output
  character(len=max_path_len), dimension(:), allocatable :: filelist
  character(len=8)                                       :: adate
  character(len=6)                                       :: atime, amonth
  logical                                                :: lexist
  integer                                                :: nfiles, i, ierr, n
  integer                                                :: jjjjmmdd, hhmiss 
  real, dimension(:,:,:), allocatable                    :: grid
  real(kind=8), dimension(:), allocatable                :: gtime, time
  real(kind=8)                                           :: jdate
  integer                                                :: numpoint
  integer                                                :: ibdate, ibtime
  integer, dimension(maxpoint)                           :: releases
  real                                                   :: bndx, bndy, delx, dely
  integer                                                :: numx, numy, numz

  ! list all grid files in directory
  if ( trim(config%filefreq).eq.'month' ) then
    write(adate,'(I6.6)') timestamp/100
    print*, adate
  else if ( trim(config%filefreq).eq.'day' ) then
    write(adate,'(I8.8)') timestamp
  endif
  path_flexpart = trim(files%path_flexpart)//trim(recname(nr))//'/'//trim(adate)//'/'
  path_output = trim(files%path_output)//trim(recname(nr))//'/'
  print*, path_flexpart
  print*, path_output
  call system('mkdir '//trim(path_output))
  if ( config%lnested ) then
    call system('ls '//trim(path_flexpart)//' | grep grid_time_nest_ | wc -l > '//trim(path_output)//'files.txt')
    call system('ls '//trim(path_flexpart)//' | grep grid_time_nest_  >> '//trim(path_output)//'files.txt')
  else
    call system('ls '//trim(path_flexpart)//' -I "*nest*" | grep grid_time_ | wc -l > '//trim(path_output)//'files.txt')
    call system('ls '//trim(path_flexpart)//' -I "*nest*" | grep grid_time_ >> '//trim(path_output)//'files.txt')
  endif

  ! read file list
  open(100,file=trim(path_output)//'files.txt',status="old",action="read",iostat=ierr)
  if ( ierr.ne.0 ) then
    print*, 'ERROR: problem read files.txt'
    stop
  endif
  read(100,*) nfiles
  allocate( filelist(nfiles) ) 
  do i = 1, nfiles
    read(100,fmt='(A)',iostat=ierr) filelist(i)
    if (ierr.ne.0) exit
  end do
  close(100)

  ! read header for this month to get ngrid
  if ( config%lnested ) then
    filename = trim(path_flexpart)//'header_nest'
    inquire(file=trim(filename),exist=lexist)
    if ( .not.lexist ) then
      ! test if is FPv11 file
      filename = trim(path_flexpart)//'header_nest_grid_time'
      inquire(file=trim(filename),exist=lexist)
    endif
  else
    filename = trim(path_flexpart)//'header'
    inquire(file=trim(filename),exist=lexist)
  endif
  if ( .not.lexist ) then
    print*, 'ERROR: cannot find flexpart header'
    stop
  endif
  print*, 'Reading flexpart header: '//trim(filename)
  call read_header(filename, numpoint, ibdate, ibtime, releases, &
                   numx, numy, numz, bndx, bndy, delx, dely)
  if ( config%linversionout ) then
    ngrid = trajdays*ndgrid+2
  else
    ngrid = numpoint
  endif
  print*, 'ngrid = ',ngrid
  allocate( grid(nxgrid,nygrid,ngrid) )
  allocate( gtime(ngrid) )
  allocate( time(ngrid) )

  ! loop over files
  filedates = trim(path_flexpart)//'dates'
  do i = 1, nfiles
    ! read binary grid files
    if ( config%lnested ) then
      adate = filelist(i)(16:23)
      if ( trim(config%filefreq).eq.'month' ) then
        atime = filelist(i)(24:29)
      else if ( trim(config%filefreq).eq.'day' ) then
        ! assume satellite -> get number of retrieval
        atime = filelist(i)(25:30)
      endif
    else
      adate = filelist(i)(11:18)
      if ( trim(config%filefreq).eq.'month' ) then
        atime = filelist(i)(19:24)
      else if ( trim(config%filefreq).eq.'day' ) then
        ! assume satellite -> get number of retrieval
        atime = filelist(i)(20:25)
      endif
    endif
    read(adate,*) jjjjmmdd
    read(atime,*) hhmiss
    print*, 'jjjjmmdd = ',jjjjmmdd
    print*, 'hhmiss = ',hhmiss
    if ( config%linversionout ) then
      jdate = juldate(jjjjmmdd, hhmiss)
    else
      jdate = juldate(datei,timei)
    endif
    filename = trim(path_flexpart)//trim(filelist(i))
    if ( config%linversionout ) then
      call read_grid(config, filename, filedates, jdate, nxgrid, nygrid, grid, gtime)
    else
      call read_grid_std(config, filename, jdate, nxgrid, nygrid, grid, gtime)
    endif
    ! write to ncdf
    do n = 1, ngrid
      call caldate(gtime(n), jjjjmmdd, hhmiss)
      time(n) = real(jjjjmmdd,kind=8)*100d0+real(hhmiss/10000,kind=8)
    end do
    if ( config%lnested ) then
      if ( trim(config%filefreq).eq.'month' ) then
        filename = trim(path_output)//'grid_time_nest_'//adate//atime//'_001.nc'
      else if ( trim(config%filefreq).eq.'day' ) then
        ! assume satellite
        filename = trim(path_output)//'grid_time_nest_'//adate//'_'//atime//'.nc'
      endif
    else
      if ( trim(config%filefreq).eq.'month' ) then
        filename = trim(path_output)//'grid_time_'//adate//atime//'_001.nc'
      else if ( trim(config%filefreq).eq.'day' ) then
        filename = trim(path_output)//'grid_time_'//adate//'_'//atime//'.nc'
      endif
    endif
    call write_ncdf(config, filename, time, grid)
  end do  

  deallocate( grid )
  deallocate( gtime )
  deallocate( time )

end subroutine grid_convert

