!---------------------------------------------------------------------------------------
! GRID_TO_NCDF: grid_to_ncdf
!---------------------------------------------------------------------------------------
!  FLEXINVERT is free software: you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation, either version 3 of the License, or
!  (at your option) any later version.
!
!  FLEXINVERT is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with FLEXINVERT.  If not, see <http://www.gnu.org/licenses/>.
!
!  Copyright 2017, Rona Thompson
!---------------------------------------------------------------------------------------
!
!> grid_to_ncdf
!! 
!! Purpose:    Converts binary grid files in sparse matrix form to NetCDF files 
!!
!---------------------------------------------------------------------------------------

program grid_to_ncdf

  use mod_flexpart
  use mod_var
  use mod_dates
  use mod_settings

  implicit none

  ! set types
  type ( files_t )                        :: files
  type ( config_t )                       :: config

  character(max_path_len)                 :: settings_files
  character(len=6)                        :: adate
  integer                                 :: nr
  integer                                 :: yyyymmdd, hhmiss, eomday
  real(kind=8)                            :: juldatei, juldatef, jd, dt

  call getarg(1,settings_files)
  if (settings_files == '') then
    stop 'ERROR: need to specify SETTINGS_files file'
  endif

  ! read file settings
  call read_settings(settings_files, files, config)

  ! initialize parameters
  call initialize(files, config)
  juldatei = juldate(config%datei, 0)
  juldatef = juldate(config%datef, 0)

  ! convert grid files
  do nr = 1, nrec
    jd = juldatei
    do while ( jd.le.juldatef )
      call caldate(jd, yyyymmdd, hhmiss)
      select case ( config%intype )
        case (0)
          ! grid_time
          call grid_convert(files, config, nr, yyyymmdd)
        case (1)
          ! grid_initial 
          call init_convert(files, config, nr, yyyymmdd)
        case (2)
          ! factor
          call factor_convert(files, config, nr, yyyymmdd)
      endselect
      if ( trim(config%filefreq).eq.'month' ) then
        eomday = calceomday(yyyymmdd/100)
        dt = dble(eomday)
      else if ( trim(config%filefreq).eq.'day' ) then
        dt = 1d0
      endif
      jd = jd + dt
    end do
  end do

end program



