!---------------------------------------------------------------------------------------
! PREP_SATELLITE: main
!---------------------------------------------------------------------------------------
!  FLEXINVERT is free software: you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation, either version 3 of the License, or
!  (at your option) any later version.
!
!  FLEXINVERT is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with FLEXINVERT.  If not, see <http://www.gnu.org/licenses/>.
!
!  Copyright 2022, Rona Thompson
!---------------------------------------------------------------------------------------
!
!> main
!!
!! Purpose:    Prepares the input files needed to run FLEXPART for satellite or other 
!!             column measurements to give a total column source-receptor-relationship 
!!
!!             Prepares the following:
!!             
!!             COMMAND   -  flexpart settings
!!             OUTGRID   -  definition of flexpart output grid
!!             AGECLASS  -  definition of age of particles
!!             releases.nc  -  definition of all releases for all retrievals
!!             auxilliary.nc - supporting info (averageing kernel)
!!
!! 
!---------------------------------------------------------------------------------------

program main

  use mod_var
  use mod_dates
  use mod_settings

  implicit none

  ! set types
  type (settings_t)        :: settings

  character(max_path_len)  :: settings_file, logfile
  integer                  :: ierr

  ! initialization
  ! --------------

  call getarg(1,settings_file)
  if (settings_file == '') then
    stop 'ERROR: need to specify SETTINGS file'
  endif

  call getarg(2,logfile)
  if (logfile == '') then
    stop 'ERROR: need to specify logfile'
  endif
  open(logid,file=trim(logfile),status='replace',action='write',iostat=ierr)  

  ! read settings
  call read_settings(settings_file, settings)

  ! get satellite data
  ! ------------------

  if ( settings%satellite.eq.'tropomi' ) then
    ! tropomi
    call get_tropomi(settings)
  else if ( settings%satellite.eq.'bremen' ) then
    ! bremen retrieval for tropomi
    call get_bremen(settings)
  else if ( settings%satellite.eq.'oco2' ) then
    call get_oco2(settings)
  else
    ! add other satellites here
    write(logid,*) 'ERROR: unknown satellite type'
    stop
  endif


end program

