!---------------------------------------------------------------------------------------
! FLEXINVERT: mod_covar
!---------------------------------------------------------------------------------------
!  FLEXINVERT is free software: you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation, either version 3 of the License, or
!  (at your option) any later version.
!
!  FLEXINVERT is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with FLEXINVERT.  If not, see <http://www.gnu.org/licenses/>.
!
!  Copyright 2017, Rona Thompson
!---------------------------------------------------------------------------------------
!
!> mod_covar
!!
!! Purpose:    Module for constructing the error covariance data structure
!!
!---------------------------------------------------------------------------------------


module mod_covar

  use mod_var

  implicit none
  private

  public :: covar_t, alloc_covar, dealloc_covar

  type :: covar_t
 
    real(kind=8), dimension(:),   allocatable :: evals   ! eigen values of error covariance matrix
    real(kind=8), dimension(:,:), allocatable :: evecs   ! eigen vectors of error covariance matrix
    real(kind=8), dimension(:,:), allocatable :: cort    ! temporal correlation matrix
    real(kind=8), dimension(:,:), allocatable :: icort   ! inverse of temporal correlation matrix
    real(kind=8), dimension(:,:), allocatable :: sqcort  ! square root of temporal correlation matrix
    real(kind=8), dimension(:,:), allocatable :: isqcort ! inverse of sqrt of temporal correlation matrix

  end type

  contains

  ! --------------------------------------------------
  ! alloc_covar
  ! --------------------------------------------------

  subroutine alloc_covar(covar)

    use mod_var 

    type (covar_t), intent(in out)       :: covar
    integer                              :: ierr

    allocate( covar%evals(neig), stat = ierr )
    if ( ierr.ne.0 ) then
      write(logid,*) 'ERROR alloc_covar: not enough memory'
      stop
    endif
    allocate( covar%evecs(ndvar,neig), stat = ierr )
    if ( ierr.ne.0 ) then
      write(logid,*) 'ERROR alloc_covar: not enough memory'
      stop
    endif
    allocate( covar%cort(ntstate,ntstate), stat = ierr )
    if ( ierr.ne.0 ) then
      write(logid,*) 'ERROR alloc_covar: not enough memory'
      stop
    endif
    allocate( covar%icort(ntstate,ntstate), stat = ierr )
    if ( ierr.ne.0 ) then
      write(logid,*) 'ERROR alloc_covar: not enough memory'
      stop
    endif
    allocate( covar%sqcort(ntstate,ntstate), stat = ierr )
    if ( ierr.ne.0 ) then
      write(logid,*) 'ERROR alloc_covar: not enough memory'
      stop
    endif
    allocate( covar%isqcort(ntstate,ntstate), stat = ierr )
    if ( ierr.ne.0 ) then
      write(logid,*) 'ERROR alloc_covar: not enough memory'
      stop
    endif

  end subroutine alloc_covar

  ! --------------------------------------------------
  ! dealloc_covar
  ! --------------------------------------------------

  subroutine dealloc_covar(covar)

    type (covar_t), intent(in out)       :: covar

    if ( allocated(covar%evals) )   deallocate(covar%evals)
    if ( allocated(covar%evecs) )   deallocate(covar%evecs)
    if ( allocated(covar%cort) )    deallocate(covar%cort)
    if ( allocated(covar%icort) )   deallocate(covar%icort)
    if ( allocated(covar%sqcort) )  deallocate(covar%sqcort)
    if ( allocated(covar%isqcort) ) deallocate(covar%isqcort)

  end subroutine dealloc_covar

  ! --------------------------------------------------

end module mod_covar




